@@ -10,8 +10,14 @@ Various Format descriptions for MagPy.
 """
 
 # import order matters - NamedTemporaryFile must be one of the first!
-__all__ = ['lib','opt','stream','absolutes','transfer','database']
+__all__ = ['lib','opt','stream','absolutes','transfer','database','gui','collector'
+]
 from magpy.stream import *
 from magpy.absolutes import *
 from magpy.transfer import *
 from magpy.database import *
+
+import lib
+import opt
+import gui
+import collector
