@@ -40,8 +40,11 @@ Version 1.0 (from the 23.02.2012)
 """
 
 from magpy.stream import *
+from magpy.database import *
 from magpy.transfer import *
 
+import dateutil.parser as dparser
+
 MAGPY_SUPPORTED_ABSOLUTES_FORMATS = ['MAGPYABS','MAGPYNEWABS','AUTODIF','UNKNOWN']
 ABSKEYLIST = ['time', 'hc', 'vc', 'res', 'f', 'mu', 'md', 'expectedmire', 'varx', 'vary', 'varz', 'varf', 'var1', 'var2']
 
@@ -291,21 +294,6 @@ class AbsoluteData(object):
 
         return np.asarray([eval('elem.'+key) for elem in self])
 
-    def _nan_helper(self, y):
-        """Helper to handle indices and logical indices of NaNs. Taken from eat (http://stackoverflow.com/questions/6518811/interpolate-nan-values-in-a-numpy-array)
-
-        Input:
-            - y, 1d numpy array with possible NaNs
-        Output:
-            - nans, logical indices of NaNs
-            - index, a function, with signature indices= index(logical_indices),
-              to convert logical indices of NaNs to 'equivalent' indices
-        Example:
-            >>> # linear interpolation of NaNs
-            >>> nans, x= _nan_helper(y)
-            >>> y[nans]= np.interp(x(nans), x(~nans), y[~nans])
-        """
-        return np.isnan(y), lambda z: z.nonzero()[0]
 
     def _insert_function_values(self, function,**kwargs):
         """
@@ -470,9 +458,17 @@ class AbsoluteData(object):
 
         #drop NANs from input stream - positions
         expmire = self._get_column('expectedmire')
+        expmire = [float(elem) for elem in expmire]
         expmire = np.mean([elem for elem in expmire if not isnan(elem)])
         poslst = [elem for elem in self if not isnan(elem.hc) and not isnan(elem.vc)]
 
+        try:
+            if len(poslst) < 1:
+                loggerabs.warning("_calcdec: could not identify measurement positions - aborting")
+                return LineStruct()
+        except:
+            loggerabs.warning("_calcdec: could not assign measurement positions - aborting")
+            return LineStruct()
 
         # Check that: Should be correct if the order of input values is correct
         miremean = np.mean([poslst[1].mu,poslst[1].md]) # fits to mathematica
@@ -538,6 +534,14 @@ class AbsoluteData(object):
             dl1.append( poslst[k].hc*np.pi/(180.0) + rescorr - variocorr[k])
             loggerabs.debug("_calcdec: Horizontal angles: %f, %f, %f" % (poslst[k].hc, rescorr, variocorr[k]))
 
+        try:
+            if len(dl1) < 8:
+                loggerabs.warning("_calcdec: horizontal angles could not be assigned - aborting")
+                return LineStruct()
+        except:
+            loggerabs.warning("_calcdec: horizontal angles could not be assigned - aborting")
+            return LineStruct()
+
         # use selected steps, default is average....
         for k in range(0,7,2):
             if usestep == 1:
@@ -1004,7 +1008,10 @@ class AbsoluteData(object):
                 print "All results: " , resultline
             # requires succesful declination determination
             if isnan(resultline.y):
-                loggerabs.error('%s : CalcAbsolutes: Declination could not be determined - aborting' % num2date(self[0].time).replace(tzinfo=None))
+                try:
+                    loggerabs.error('%s : CalcAbsolutes: Declination could not be determined - aborting' % num2date(self[0].time).replace(tzinfo=None))
+                except:
+                    loggerabs.error('unkown : CalcAbsolutes: Declination could not be determined - aborting')
                 break
             # use incstart and ystart as boundary conditions
             try: # check, whether outline already exists
@@ -1015,16 +1022,29 @@ class AbsoluteData(object):
             except:
                 inc = incstart
             outline, xstart, ystart = self._calcinc(resultline,scalevalue=scalevalue,incstart=inc,deltaI=deltaI,iterator=i,usestep=usestep,annualmeans=annualmeans)
+            
             if debugmode:
                 print "Calculated I (%f) - iteration step %d" %(outline[1],i)
 
-        loggerabs.info('%s : Declination: %s, Inclination: %s, H: %.1f, F: %.1f' % (num2date(outline.time), deg2degminsec(outline.y),deg2degminsec(outline.x),outline.f*np.cos(outline.x*np.pi/180),outline.f))
 
-        if printresults:
-            print 'Vector:'
-            print 'Declination: %s, Inclination: %s, H: %.1f, F: %.1f' % (deg2degminsec(outline.y),deg2degminsec(outline.x),outline.f*np.cos(outline.x*np.pi/180),outline.f)
-            print 'Collimation and Offset:'
-            print 'Declination:    S0: %.3f, delta H: %.3f, epsilon Z: %.3f\nInclination:    S0: %.3f, epsilon Z: %.3f\nScalevalue: %.3f' % (outline.var1,outline.var2,outline.var3,outline.var4,outline.var5,outline.t2)
+        #test whether outline is a linestruct object - if not use an empty object
+        try:
+            test = outline.x
+
+            loggerabs.info('%s : Declination: %s, Inclination: %s, H: %.1f, F: %.1f' % (num2date(outline.time), deg2degminsec(outline.y),deg2degminsec(outline.x),outline.f*np.cos(outline.x*np.pi/180),outline.f))
+
+            if printresults:
+                print 'Vector:'
+                print 'Declination: %s, Inclination: %s, H: %.1f, F: %.1f' % (deg2degminsec(outline.y),deg2degminsec(outline.x),outline.f*np.cos(outline.x*np.pi/180),outline.f)
+                print 'Collimation and Offset:'
+                print 'Declination:    S0: %.3f, delta H: %.3f, epsilon Z: %.3f\nInclination:    S0: %.3f, epsilon Z: %.3f\nScalevalue: %.3f' % (outline.var1,outline.var2,outline.var3,outline.var4,outline.var5,outline.t2)
+
+        except:
+            text = 'calcabsolutes: invalid LineStruct Object returned from calcinc function'
+            loggerabs.warning(text)
+            if printresults:
+                print text
+            outline = LineStruct()
 
         return outline
 
@@ -1157,6 +1177,12 @@ def _absRead(filename, dataformat=None, headonly=False, **kwargs):
 
     stream = readAbsFormat(filename, format_type, headonly=headonly, **kwargs)
 
+    # If stream could not be read return an empty datastream object
+    try:
+        xxx = len(stream)
+    except:
+        stream = DataStream()
+
     return stream
 
 
@@ -1388,8 +1414,11 @@ def analyzeAbsFiles(debugmode=None,**kwargs):
         iterationtime = date2num(num2date(iterationtime).replace(tzinfo=None)+timedelta(days=30))    
         for stream in testlst:
             #print len(stream)
-            lengthoferrorsbefore = _logfile_len('magpy.log','ERROR')
+            lengthoferrorsbefore = _logfile_len(logfile,'ERROR')
 
+            # XXX ###################################
+            # XXX   Remove the reference to plog !!!
+            # XXX ###################################
             cnt += 1
             plog.addcount(cnt, len(localfilelist))
             # ######## Process counter
@@ -1433,7 +1462,7 @@ def analyzeAbsFiles(debugmode=None,**kwargs):
                     result.str3 = scalarinst
 
                 # Get the amount of error messages after the analysis
-                lengthoferrorsafter = _logfile_len('magpy.log','ERROR')
+                lengthoferrorsafter = _logfile_len(logfile,'ERROR')
 
                 if lengthoferrorsafter > lengthoferrorsbefore:
                     movetoarchive = False
@@ -1545,6 +1574,427 @@ def analyzeAbsFiles(debugmode=None,**kwargs):
     return st
 
 
+
+def absoluteAnalysis(absdata, variodata, scalardata, **kwargs): 
+    """
+    DEFINITION:
+        Analyze absolute data from files or database and create datastream
+
+    PARAMETERS:
+    Variables:
+        - header_dict:  (dict) dictionary with header information 
+    Kwargs:
+        - starttime:    (string/datetime) define begin
+        - endtime:      (string/datetime) define end
+        - abstype:      (string) default manual, can be autodif
+        - db:   	(mysql database) defined by MySQLdb.connect().
+        - dbadd:        (bool) if True DI-raw data will be added to the database
+        - alpha:        (float) orientation angle 1 in deg (if z is vertical, alpha is the horizontal rotation angle)
+        - beta:         (float) orientation angle 2 in deg 
+        - deltaF:       (float) difference between scalar measurement point and pier (TODO: check the sign)
+        - diid:         (string) identifier (id) of di files (e.g. A2_WIC.txt)
+        - outputformat: (string) one of 'idf', 'xyz', 'hdf'
+        - usestep:      (int) which step to use for analysis, usually both, in autodif only 2
+        - annualmeans:  (list) provide annualmean for x,y,z as [x,y,z] with floats
+        - azimuth:      (float) required for Autodif measurements
+        - expD:         (float) expected Declination - failure produced when D differs by more than expT deg
+        - expI:         (float) expected Inclination - failure produced when I differs by more than expT deg
+        - expT:         (float) expected value threshold - default 1 deg
+        - movetoarchive:(string) define a local directory to store archived data (only works when reading files)
+    RETURNS:
+        --
+    EXAMPLE:
+        (1) >>> stream = absoluteAnalysis('/home/leon/Dropbox/Daten/Magnetism/DI-WIC/autodif',variopath,'',abstype='autodif',azimuth=267.4242,starttime='2014-02-10',endtime='2014-02-25')
+        (2) >>> stream = absoluteAnalysis('DIDATA_WIK',variopath,'',starttime='2013-12-20',endtime='2013-12-30',db=db)
+        (3) >>> stream = absoluteAnalysis('/home/leon/Dropbox/Daten/Magnetism/DI-WIC/raw/',variopath,scalarpath,diid='A2_WIC.txt',starttime='2014-02-10',endtime='2014-02-25',db=db,dbadd=True)
+        (3) >>> stream = absoluteAnalysis('http://localhost/mydata.html',variopath,scalarpath,diid='A2_WIC.txt',db=db,dbadd=True)
+
+    PERFORMED TESTS:
+        (OK)1. Read data from database and files
+        (OK)2. Read data from autodif and manual analysis 
+        (OK)3. Put data to database (dbadd) 
+        (OK)4. Check parameter for all possibilities: a) no db, b) db, c) db and override by input
+        (OK)5. Archiving function
+        6. Appropriate information on failed analyses -> add test for expected values 
+        (OK)7. is the usestep variable correctly applied for autodif and normal?
+        (OK)8. overwrite of existing database lines?
+        (OK)9. Order of saving data when analyzing older data sets - requires reload and delete
+        10. Test memory capabilities for large data sets (~ 6month with two varios, one scalar, no vario and scalar data)
+        (OK)11. Check URL/FTP read, single file and directory read
+        
+        Bugs: 
+        when importing db2diline from database, the DILineStruct, defined in absolutes is missing.... why???????? see Test4 in FullAbsoluteAnalysis -- order of import is very important
+      
+    """
+    db = kwargs.get('db')
+    dbadd = kwargs.get('dbadd')
+    alpha = kwargs.get('alpha')
+    starttime = kwargs.get('starttime')
+    endtime = kwargs.get('endtime')
+    beta = kwargs.get('beta')
+    stationid = kwargs.get('stationid')
+    pier = kwargs.get('pier')
+    deltaF = kwargs.get('deltaF')
+    diid = kwargs.get('diid')
+    outputformat = kwargs.get('outputformat')
+    usestep = kwargs.get('usestep')
+    annualmeans = kwargs.get('annualmeans')
+    azimuth = kwargs.get('azimuth') # 267.4242 # A16 to refelctor
+    abstype = kwargs.get('abstype')
+    expT = kwargs.get('expT')
+    expI = kwargs.get('expI')
+    expD = kwargs.get('expD')
+    movetoarchive = kwargs.get('movetoarchive')
+
+    if not outputformat:
+        outputformat='idf'
+    if not annualmeans:
+        annualmeans=[20000,1200,43000]
+    if not abstype:
+        abstype = "manual"
+    if abstype == 'autodif': # we also check the person given in the file. if this is AutoDIF abstype will be set correctly automatically
+        usestep=2
+        if not azimuth:
+            print "Azimuth needs ro be provided for AutoDIF measurements"
+            return
+    if not expT:
+        expT = 1
+
+    # ####################################
+    # 1. Get parameters from db or input (input overrides)
+    # ####################################
+    if db: 
+        cursor = db.cursor()
+        # Get all othere relevant data first before trying to read table (only if not provided by the user)
+        try:
+            if not stationid:
+                stationid =  dbgetstring(db, 'DATAINFO', vario, 'StationID')
+        except:
+            pass
+        try:
+            if not pier:
+                pier = dbgetstring(db, 'DATAINFO', vario, 'DataDeltaReferencePier')
+        except:
+            pass
+        try:
+            if not alpha:
+                alpha =  dbgetfloat(db, 'DATAINFO', vario, 'DataSensorAzimuth')
+        except:
+            pass
+        try:
+            if not beta:
+                beta =  dbgetfloat(db, 'DATAINFO', vario, 'DataSensorTilt')
+        except:
+            pass
+        try:
+            if not deltaF:
+                deltaF =  dbgetfloat(db, 'DATAINFO', scalar, 'DataDeltaF')
+        except:
+            pass
+
+    if not alpha: 
+        alpha=0.0
+    if not beta:
+        beta=0.0
+    if not deltaF:
+        deltaF=0.0
+    if not stationid:
+        stationid=''
+    if not pier:
+        pier=''
+    if not diid:
+        diid=".txt"
+
+    # Please Note for pier information always the existing pier in the file is used
+    print "Using the following parameters (alpha, beta, stationid, pier, deltaF, diid):", alpha, beta, stationid, pier, deltaF, diid
+
+    # ####################################
+    # 2. Get absolute data
+    # ####################################
+
+    # 2.1 Database or File -> Get a datelist
+    # --------------------
+    readfile = True
+    filelist, datelist = [],[]
+    failinglist = []
+    successlist = []
+    if db:        
+        # Check whether absdata exists as table
+        cursor.execute("SHOW TABLES LIKE '%s'" % absdata)
+        try:
+            value = cursor.fetchone()[0]
+            # Found a table - now reading it and setting file read to zero
+            try:
+                sql = "SELECT StartTime FROM " + value
+                cursor.execute(sql)
+                values = cursor.fetchall()
+                lst = [elem[0].split()[0] for elem in values]
+                datelist = list(set(lst))
+            except:
+                print "absoluteAnalysis: Error while getting Absdata from db"
+                return
+            # DI TABLE FOUND
+            readfile = False
+        except:
+            pass
+        
+    if readfile:
+        # Get list of files
+        if "://" in absdata:
+            print "Found URL code - requires name of data set with date" 
+            filelist.append(absdata)
+            movetoarchive = False # XXX No archiving function supported so far - will be done as soon as writing to files is available
+        elif os.path.isfile(absdata):
+            print "Found single file"
+            filelist.append(absdata)
+        else:
+            for file in os.listdir(absdata):
+                if file.endswith(diid):
+                    filelist.append(os.path.join(absdata,file))
+
+        for elem in filelist:
+            head, tail = os.path.split(elem)
+            try:
+                date = dparser.parse(tail,fuzzy=True)
+                datelist.append(datetime.strftime(date,"%Y-%m-%d"))
+            except:
+                print "absoluteAnalysis: Found date problem in file: %s" % tail
+                failinglist.append(elem)
+
+        datelist = list(set(datelist))
+
+
+    datetimelist = [datetime.strptime(elem,'%Y-%m-%d') for elem in datelist]
+
+    empty = DataStream()
+    if starttime:
+        datetimelist = [elem for elem in datetimelist if elem >= empty._testtime(starttime)]
+    if endtime:
+        datetimelist = [elem for elem in datetimelist if elem <= empty._testtime(endtime)]
+
+    if not len(datetimelist) > 0:
+        print "absoluteAnalysis: No matching dates found - aborting"
+        return
+
+    # 2.2 Cycle through datetimelist 
+    # --------------------
+    # read varios, scalar and all absfiles of one day
+    # analyze data for each day and append results to a resultstream    
+    # XXX possible issues: an absolute measurement which is performed in two day (e.g. across midnight)
+    resultstream = DataStream()
+    for date in sorted(datetimelist):
+        variofound = True
+        scalarfound = True
+        print ""
+        print "------------------------------------------------------"
+        print "Starting analysis for ", date
+        print "------------------------------------------------------"
+        # a) Read variodata
+        variostr = read(variodata,starttime=date,endtime=date+timedelta(days=1))
+        print "Length of Variodata:", len(variostr)
+        if len(variostr) > 3: # can contain ([], 'File not specified')
+            variostr =variostr.rotation(alpha=alpha, beta=beta)
+            vafunc = variostr.interpol(['x','y','z'])
+        else:
+            print "absoluteAnalysis: no variometer data available"
+            variofound = False
+            
+        # b) Load Scalardata
+        print "-----------------"
+        scalarstr = read(scalardata,starttime=date,endtime=date+timedelta(days=1))
+        print "Length of Scalardata:", len(scalarstr)
+        if len(scalarstr) > 3: # Because scalarstr can contain ([], 'File not specified')
+            scfunc = scalarstr.interpol(['f'])
+        else:
+            print "absoluteAnalysis: no scalar data available"
+            scalarfound = False
+
+        # c) get absolute data
+        abslist = []
+        if readfile:
+            datestr = datetime.strftime(date,"%Y-%m-%d")
+            datestr2 = datetime.strftime(date,"%Y%m%d") # autodif
+            print "-----------------"
+            #print datestr
+            if abstype == 'autodif':
+                difiles = [di for di in filelist if datestr2 in di]
+            else:
+                difiles = [di for di in filelist if datestr in di]
+            if len(difiles) > 0:
+                for elem in difiles:
+                    absst = absRead(elem,azimuth=azimuth,pier=pier,output='DIListStruct')
+                    #print "LENGTH:",len(absst)
+                    try:
+                        if not len(absst) > 1:
+                            stream = absst[0].getAbsDIStruct()
+                            abslist.append(absst)
+                            if db and dbadd:
+                                diline2db(db, absst,mode='insert',tablename='DIDATA_'+stationid)
+                        else:
+                            for a in absst:
+                                stream = a.getAbsDIStruct()
+                                abslist.append(a)
+                            if db and dbadd:
+                                diline2db(db, absst,mode='insert',tablename='DIDATA_'+stationid)
+                        #print "absoluteAnalysis: Successful analyse of %s" % elem 
+                        successlist.append(elem)
+                    except:
+                        print "absoluteAnalysis: Failed to analyse %s - problem of filestructure" % elem 
+                        failinglist.append(elem)
+                        # TODO Drop that line from filelist
+                        pass
+        else:
+            #get list from database
+            startd = datetime.strftime(date,"%Y-%m-%d")
+            endd = datetime.strftime(date+timedelta(days=1),"%Y-%m-%d")
+            #pier = 'D'
+            #stationid = 'WIK'
+            sql = "Pier='%s'" % pier
+            tablename = 'DIDATA_%s' % stationid
+            #print sql, tablename
+            abslist = db2diline(db,starttime=startd,endtime=endd,sql=sql,tablename=tablename)
+
+        for absst in abslist:
+            print "-----------------"
+            print "Analyzing %s measurement from %s" % (abstype,str(date))
+            #if readfile:
+            try:
+                stream = absst[0].getAbsDIStruct()
+            except:
+                stream = absst.getAbsDIStruct()
+            # if usestep not given and AutoDIF measruement found
+            if stream[0].person == 'AutoDIF' and not usestep:
+                usestep = 2
+            print "USESTEP:", usestep
+            if variofound:
+                stream = stream._insert_function_values(vafunc)
+            if scalarfound:
+                stream = stream._insert_function_values(scfunc,funckeys=['f'],offset=deltaF)
+            result = stream.calcabsolutes(usestep=usestep,annualmeans=annualmeans,printresults=True,debugmode=False)
+            dataok = True
+            if expD:
+                if not expD-expT < result.y < expD+expT:
+                    test = datetime.strftime(num2date(stream[0].time),'%Y-%m-%d_%H-%M-%S')
+                    xl = [ el for el in difiles if test in el]
+                    failinglist.append(xl[0])
+                    print "absoluteAnalysis: Failed to analyse %s - threshold for acceptable angular offset exceeded" % test
+                    dataok = False
+            if expI and dataok:
+                if not expI-expT < result.x < expI+expT:
+                    test = datetime.strftime(num2date(stream[0].time),'%Y-%m-%d_%H-%M-%S')
+                    xl = [ el for el in difiles if test in el]
+                    failinglist.append(xl[0])
+                    print "absoluteAnalysis: Failed to analyse %s - threshold for acceptable angular offset exceeded" % test
+                    dataok = False
+            if dataok:
+                resultstream.add(result)
+
+    # ####################################
+    # 3. Format output
+    # ####################################
+
+    # 3.1 Header information 
+    # --------------------
+    resultstream.header['col-time'] = 'Epoch'
+    resultstream.header['col-x'] = 'i'
+    resultstream.header['unit-col-x'] = 'deg'
+    resultstream.header['col-y'] = 'd'
+    resultstream.header['unit-col-y'] = 'deg'
+    resultstream.header['col-z'] = 'f'
+    resultstream.header['unit-col-z'] = 'nT'
+    resultstream.header['col-f'] = 'f'
+    resultstream.header['col-dx'] = 'basex'
+    resultstream.header['col-dy'] = 'basey'
+    resultstream.header['col-dz'] = 'basez'
+    resultstream.header['col-df'] = 'dF'
+    resultstream.header['col-t1'] = 'T'
+    resultstream.header['col-t2'] = 'ScaleValueDI'
+    resultstream.header['col-var1'] = 'Dec_S0'
+    resultstream.header['col-var2'] = 'Dec_deltaH'
+    resultstream.header['col-var3'] = 'Dec_epsilonZ'
+    resultstream.header['col-var4'] = 'Inc_S0'
+    resultstream.header['col-var5'] = 'Inc_epsilonZ'
+    resultstream.header['col-str1'] = 'Person'
+    resultstream.header['col-str2'] = 'DI-Inst'
+    resultstream.header['col-str3'] = 'F-Inst'
+    resultstream.header['col-str4'] = 'Vario-Inst'
+
+    if outputformat == 'xyz':
+        #for elem in st:
+        result = result.idf2xyz()
+        result.typ = 'xyzf'         
+        resultstream.header['col-x'] = 'x'
+        resultstream.header['unit-col-x'] = 'nT'
+        resultstream.header['col-y'] = 'y'
+        resultstream.header['unit-col-y'] = 'nT'
+        resultstream.header['col-z'] = 'z'
+        resultstream.header['unit-col-z'] = 'nT'
+    elif outputformat == 'hdz':
+        #for elem in st:
+        result = result.idf2xyz()
+        #for elem in st:
+        result = result.xyz2hdz()
+        result.typ = 'hdzf'         
+        resultstream.header['col-x'] = 'h'
+        resultstream.header['unit-col-x'] = 'nT'
+        resultstream.header['col-y'] = 'd'
+        resultstream.header['unit-col-y'] = 'deg'
+        resultstream.header['col-z'] = 'z'
+        resultstream.header['unit-col-z'] = 'nT'
+
+    #print "Files for archive:"
+    #print "---------------------------"
+    archivelist = [x for x in successlist if x not in failinglist]
+    print "------------------------------------------------------"
+    print "Failed files:"
+    print "------------------------------------------------------"
+    if len(failinglist) > 0:
+        for el in failinglist:
+            print el
+    else:
+        print "None"
+    print "------------------------------------------------------"
+
+
+    # 3.2 Eventually archive the data 
+    # --------------------
+    # XXX possible issues: direct ftp reading or url reading
+    if movetoarchive:
+        if readfile:
+            for fi in archivelist:
+                if not "://" in fi: 
+                    src = fi
+                    fname = os.path.split(src)[1]
+                    dst = os.path.join(movetoarchive,fname)
+                    shutil.move(src,dst)
+                else:
+                    fname = fi.split('/')[-1]
+                    suffix = fname.split('.')[-1]
+                    passwdtyp = fi.split(':')
+                    typus = passwdtyp[0]
+                    port = 21
+                    passwd = passwdtyp[2].split('@')[0]
+                    restpath = passwdtyp[2].split('@')[1]
+                    myproxy = restpath.split('/')[0]
+                    ftppath = restpath.split('/')[1]
+                    login = passwdtyp[1].split('//')[1]
+                    dst = os.path.join(archivepath,fname)
+                    fh = NamedTemporaryFile(suffix=suffix,delete=False)
+                    print "Fi: ", fi
+                    fh.write(urllib2.urlopen(fi).read())
+                    fh.close()
+                    shutil.move(fh.name,dst)
+                    if (typus == 'ftp'):
+                        ftpremove (ftppath=ftppath, filestr=fname, myproxy=myproxy, port=port, login=login, passwd=passwd)
+
+    resultstream = resultstream.sorting()
+
+    return resultstream
+
+
+
+
+
 def getAbsFilesFTP(**kwargs):
     """
     Obtain files from the selected directory
