@@ -24,7 +24,7 @@ def isPYCDF(filename):
     except:
         return False
 
-    loggerlib.info("format_magpy: Found PYCDF file %s" % filename)
+    loggerlib.debug("format_magpy: Found PYCDF file %s" % filename)
     return True
 
 
@@ -39,7 +39,7 @@ def isPYSTR(filename):
     if not temp.startswith(' # MagPy - ASCII'):
         return False
 
-    loggerlib.info("format_magpy: Found PYSTR file %s" % filename)
+    loggerlib.debug("format_magpy: Found PYSTR file %s" % filename)
     return True
 
 
@@ -54,7 +54,7 @@ def isPYBIN(filename):
     if not temp.startswith('# MagPyBin'):
         return False
 
-    loggerlib.info("format_magpy: Found PYBIN file %s" % filename)
+    loggerlib.debug("format_magpy: Found PYBIN file %s" % filename)
     return True
 
 
@@ -81,9 +81,13 @@ def readPYSTR(filename, headonly=False, **kwargs):
         elif elem[0]=='#':
             # blank header
             pass
-        elif elem[0]==' #':
-            # attributes
-            pass
+        elif elem[0].startswith(' #') and not elem[0].startswith(' # MagPy - ASCII'):
+            # attributes - assign header values
+            headlst = elem[0].strip(' # ').split(':')
+            headkey = headlst[0]
+            headval = headlst[1]
+            if not headkey.startswith('Column'):
+                headers[headkey] = headval
         elif elem[0]=='Epoch[]':
             for i in range(len(elem)):
                 headval = elem[i].split('[')                
@@ -305,6 +309,8 @@ def readPYBIN(filename, headonly=False, **kwargs):
     endtime = kwargs.get('endtime')
     getfile = True
 
+    loggerlib.info('readPYBIN: Reading Magpy binary data.')
+
     stream = DataStream([],{})
 
     theday = extractDateFromString(filename)
@@ -515,7 +521,9 @@ def writePYCDF(datastream, filename, **kwargs):
             key = 'Epoch'
             mycdf[key] = np.asarray([num2date(elem).replace(tzinfo=None) for elem in col])
         elif len(col) > 0:
-            mycdf[key] = col
+            nonetest = [elem for elem in col if not elem == None]
+            if len(nonetest) > 0:
+                mycdf[key] = col
         for keydic in headdict:
             if keydic == ('col-'+key):
                 try:
@@ -529,3 +537,56 @@ def writePYCDF(datastream, filename, **kwargs):
                     pass
                     
     mycdf.close()
+
+
+def writePYASCII(datastream, filename, **kwargs):
+    """
+    Function to write basic ASCII data 
+    """
+
+    mode = kwargs.get('mode')
+    loggerlib.info("writePYASCII: Writing file to %s" % filename)
+
+    if os.path.isfile(filename):
+        if mode == 'skip': # skip existing inputs
+            exst = read(path_or_url=filename)
+            datastream = mergeStreams(exst,datastream,extend=True)
+            myFile= open( filename, "wb" )
+        elif mode == 'replace': # replace existing inputs
+            exst = read(path_or_url=filename)
+            datastream = mergeStreams(datastream,exst,extend=True)
+            myFile= open( filename, "wb" )
+        elif mode == 'append':
+            myFile= open( filename, "ab" )
+        else:
+            myFile= open( filename, "wb" )
+    else:
+        myFile= open( filename, "wb" )
+    wtr= csv.writer( myFile )
+    headdict = datastream.header
+    head, line = [],[]
+
+    keylst = datastream._get_key_headers()
+    keylst[:0] = ['time']
+
+    if not mode == 'append':
+        for key in keylst:
+            title = headdict.get('col-'+key,'-') + '[' + headdict.get('unit-col-'+key,'') + ']'
+            head.append(title)
+        head[0] = 'Time'
+        wtr.writerow( [' # ASCII'] )
+        wtr.writerow( head )
+    for elem in datastream:
+        row = []
+        for key in keylst:
+            if key.find('time') >= 0:
+                try:
+                    row.append( datetime.strftime(num2date(eval('elem.'+key)).replace(tzinfo=None), "%Y-%m-%dT%H:%M:%S.%f") )
+                except:
+                    row.append( float('nan') )
+                    pass
+            else:
+                row.append(eval('elem.'+key))
+        wtr.writerow( row )
+    myFile.close()
+
