@@ -5,9 +5,9 @@ Written by Roman Leonhardt June 2012
 - contains test and read function, toDo: write function
 """
 
-from stream import *
+from magpy.stream import *
 # absolute data / DI
-from lib.format_abs_magpy import *
+from magpy.lib.format_abs_magpy import *
 
 def isAbsFormat(filename, format_type):
     if (format_type == "MAGPYABS"):
